<?php

namespace Drupal\media_entity_twig\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\media_library\Form\AddFormBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for adding Twig-based media items to the media library.
 */
class TwigMediaLibraryAddForm extends AddFormBase {

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   Returns an instance of the form.
   */
  public static function create(ContainerInterface $container) {
    /** @var static $instance */
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_entity_twig_media_library_add';
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string,mixed> $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<string,mixed>
   *   The form structure with input elements for the media item.
   */
  protected function buildInputElement(array $form, FormStateInterface $form_state): array {
    $form['container'] = [
      '#type' => 'container',
    ];

    $form['container']['name'] = [
      '#type' => 'textfield',
      '#weight' => 1,
      '#title' => $this->t('Name'),
      '#required' => TRUE,
      '#maxlength' => 255,
    ];

    $form['container']['twig'] = [
      '#type' => 'codemirror',
      '#weight' => 1,
      '#title' => $this->t('Twig template source'),
      '#required' => TRUE,
      '#rows' => 5,
      '#twig_editor_toolbar' => TRUE,
      '#twig_editor_line_wrapping' => FALSE,
      '#twig_editor_line_numbers' => TRUE,
      '#twig_editor_fold_gutter' => FALSE,
      '#twig_editor_auto_close_tags' => TRUE,
      '#twig_editor_style_active_line' => FALSE,
      '#codemirror' => [
        'mode' => 'html_twig',
        'toolbar' => TRUE,
        'lineWrapping' => FALSE,
        'lineNumbers' => TRUE,
        'foldGutter' => FALSE,
        'autoCloseTags' => TRUE,
        'styleActiveLine' => FALSE,
      ],
      '#default_value' => '',
    ];

    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => 100,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create media item'),
      '#button_type' => 'primary',
      '#submit' => ['::addButtonSubmit'],
      '#ajax' => [
        'callback' => '::updateFormCallback',
        'wrapper' => 'media-library-wrapper',
        'url' => Url::fromRoute('media_library.ui'),
        'options' => [
          'query' => $this->getMediaLibraryState($form_state)->all() + [
            'ajax_form' => TRUE,
          ],
        ],
      ],
    ];

    return $form;
  }

  /**
   * Custom submit handler for the add form.
   *
   * @param array<string,mixed> $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function addButtonSubmit(array $form, FormStateInterface $form_state): void {
    $name = $form_state->getValue('name');
    $twig_value = $form_state->getValue('twig');

    $media = $this->entityTypeManager->getStorage('media')->create([
      'bundle' => 'twig',
      'name' => $name,
      'field_media_entity_twig' => [['value' => $twig_value]],
    ]);
    $media->save();

    $form_state->set('media', [$media]);
    $form_state->setRebuild(TRUE);
  }

}
